DialogBoxMorph subclass: #DebuggerFrameMorph
	instanceVariableNames: 'resizeMorph resizeOffset contentsFrame contentsMorph scroller thread varButton returnValue '
	classVariableNames: 'UserPos UserSize '
	poolDictionaries: ''
	category: 'Scratch-BYOB'!

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
centerOnScreen
	super centerOnScreen.
	self updateResizeMorph! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/29/2010 23:00'!
createButtons
	"Replace my current buttons with the ones specified."
	"Note: The button row morph was created by initialize; I just replace its contents with a new collection of buttons."

	| spacer jump cb |
	spacer _ Morph new extent: 10@5; color: Color transparent.
	buttonRow removeAllMorphs.

	buttonRow addMorphBack: (self buttonLabel: '>' action: #resume).

	buttonRow addMorphBack: (self buttonLabel: '>|' action: #stepNext).

	buttonRow addMorphBack: (jump _ (self buttonLabel: '>>' action: #jumpOver)).
	buttonRow addMorphBack: spacer fullCopy.

	buttonRow addMorphBack: (varButton _ (self buttonLabel: 'variables' localized action: #varMenu)).
	buttonRow addMorphBack: spacer fullCopy.

	buttonRow submorphs do: [:b |
		(b isKindOf: ResizableToggleButton2) ifTrue: [tabFields add: b]].

	cb _ thread currentBlock.
	cb nextBlock ifNil: [^jump isHidden: true].
	({CBlockMorph. IfElseBlockMorph. CustomCommandBlockMorph. CustomReporterBlockMorph} 
		includes: cb class) 
			ifFalse: [jump isHidden: true]! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/20/2010 17:43'!
extent: aPoint
	
	| newExtent min|

	min _ self class minExtent.
"	contentsMorph ifNotNil: [
		min _ min + contentsMorph extent + (0 @ 5)]."

	newExtent _ aPoint max: min.

	bounds extent = newExtent ifFalse: [
		self changed.
		bounds _ bounds topLeft extent: newExtent.
		self layoutChanged.
		self changed].

	self fixLayoutForExtent: newExtent.

	self keepOnScreen
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/20/2010 17:38'!
fixLayoutForExtent: aPoint

	super fixLayoutForExtent: aPoint.

	titleBin ifNotNil: [ titleBin width: aPoint x - 40 ].


	contentsFrame ifNil: [^self ].

	contentsFrame extent: aPoint - (65 @ 105).
	scroller extent: contentsFrame extent.
"
	contentsMorph position: contentsFrame position + (contentsFrame extent - contentsMorph extent // 2).
"

	self updateResizeMorph! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:52'!
getUserResponse
	"Wait for the user to respond, then report their response. The symbol #cancel is returned if the user wishes to cancel the operation. True/false is returned if they respond yes/no, and a string is returned if this this a string input dialog and they don't cancel."
	"Details: This is invoked synchronously from the caller. In order to keep processing inputs and updating the screen while waiting for the user to respond, this method has its own version of the World's event loop."

	| w |
	self openInWorld.
	UserPos isNil 
		ifTrue: [ self centerOnScreen ]
		ifFalse: [self snapToUserDimensions ].
		
	w _ self world.
	self buttons size > 0 ifTrue: [
		w activeHand newKeyboardFocus: self buttons last].
	tabFields size > 0 ifTrue: [
		w activeHand newKeyboardFocus: (tabFields at: 1)].

	done _ false.
	[done] whileFalse: [w doOneCycle].  "wait for user to press a button, this effectively pauses all other threads as well -jens"

	^ response
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
handlesMouseDown: evt
	^true! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/21/2010 02:59'!
hasVars

	| vars env |

	returnValue ifNotNil: [^ true].
	
	(thread currentBlock receiver isKindOf: VariableFrame)
		ifTrue: [vars _ thread currentBlock receiver ]
		ifFalse: [vars _ thread temporaryVariables ].

	vars vars keys size > 0 ifTrue: [^true].

	env _ thread currentEnvironment.
	env ifNotNil: [ env = vars ifFalse: [
		env vars keys size > 0 ifTrue: [^ true]]].

	^ false! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/20/2010 17:48'!
initialize

	super initialize.
	self title: 'Debugger'.
	resizeMorph _ ScratchResizeMorph ofType: #corner.

	scroller _ ScrollFrameMorph2 new
		growthFraction: 0.0;
		color: Color transparent "ScratchFrameMorph scriptsPaneColor".

	contentsMorph := ImageMorph new.
	contentsFrame := Morph new color: Color transparent; extent: 150 @ 80.
	scroller extent: contentsFrame extent.
	contentsFrame addMorph: scroller.
	scroller contents: contentsMorph.

	mainColumn addMorph: contentsFrame.

	self addMorph: resizeMorph.


! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/9/2010 23:54'!
jumpOver
	thread jumpOver.
	self yes! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
justDroppedInto: aMorph event: evt
	"Never embed myself in a morph (e.g. the Stage)."

	super justDroppedInto: aMorph event: evt.
	self keepOnScreen
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
keepOnScreen

	| min |
	min _ self class minExtent.

	self left < 0 ifTrue: [ self left: 0].
	self top < 0 ifTrue: [ self top: 0].

	self right > Display extent x
		ifTrue: [ resizeOffset 
			ifNil: [ 
				self left: ((Display extent x - self width) max: 0).
				(self right > Display extent x and: [self width > min x])
					ifTrue: [self width: Display extent x - self left ]]
			ifNotNil: [self width: Display extent x - self left]].

	self bottom > Display extent y
		ifTrue: [ resizeOffset 
			ifNil: [	
				self top: ((Display extent y - self height) max: 0). 
				(self bottom > Display extent y and: [self height > min y])
					ifTrue: [self height: Display extent y - self top]]
			ifNotNil: [self height: Display extent y - self top]].

! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
mouseDown: evt
	"Handle a mouse click. Left button either drags or performs click action. Right button brings up a menu."

	| p |
	p _ evt cursorPoint.
	((resizeMorph bounds expandBy: 4) containsPoint: p)
		ifFalse: [evt hand grabMorph: self]
		ifTrue: [resizeOffset _ self bottomRight - p]
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
mouseMove: evt

	resizeOffset ifNotNil: [
		self extent: (evt cursorPoint - self topLeft) + resizeOffset ]

! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
mouseUp: evt

	resizeOffset _ nil.
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/8/2010 20:41'!
on: aScratchProcess

	thread _ aScratchProcess.

	self 
		createButtons;
		updateContents;
		getUserResponse

! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/8/2010 20:41'!
on: aScratchProcess returnValue: obj

	thread _ aScratchProcess.
	returnValue _ obj.

	self 
		createButtons;
		updateContents;
		getUserResponse

! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:50'!
rememberUserDimensions
	self class setUserPos: self position size: self extent! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/20/2010 11:50'!
resume
	thread resume.
	self yes! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:51'!
snapToUserDimensions

	self extent: UserSize.
	UserPos ifNotNil: [
		self position: UserPos ].
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
step
	resizeOffset notNil | (owner isKindOf: HandMorph)
		ifTrue: [^self ].
	self keepOnScreen 
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/20/2010 11:49'!
stepNext
	thread nextStep.
	self yes! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/10/2010 00:47'!
updateContents

	| block rcvr lbl |
	block _ thread currentBlock.

	contentsMorph ifNil: [^self].
	block ifNotNil: [
		rcvr _ block receiver.
		(rcvr respondsTo: #objName)
			ifTrue: [lbl _ rcvr objName]
			ifFalse: [ 
				lbl _ 'Debugger' localized.
				(block respondsTo: #variable) ifTrue: [
					lbl _ block variable]].
		returnValue isNil
			ifTrue: [ self title: lbl]
			ifFalse: [ self title: lbl, ' ^'].
		contentsMorph form: block imageForm ].

	self hasVars ifFalse: [
		varButton isHidden: true ]
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 5/19/2010 23:32'!
updateResizeMorph
	resizeMorph ifNotNil: [
		resizeMorph position: self bottomRight - "resizeMorph extent - " bottomRightForm width ]
! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/29/2010 23:00'!
varMenu

	| menu vars env mh mw |
	menu _ CustomMenu new.

	returnValue ifNotNil: [
		menu add: 'report value' localized toolTip: returnValue action: nil.
		menu addLine].
	
	(thread currentBlock receiver isKindOf: VariableFrame)
		ifTrue: [vars _ thread currentBlock receiver ]
		ifFalse: [vars _ thread temporaryVariables ].

	vars vars keys asSortedCollection do: [:vname |
		menu add: vname toolTip: (vars vars at: vname) action: nil].

	menu addLine. 

	env _ thread currentEnvironment.
	env ifNotNil: [ env = vars ifFalse: [
		env vars keys asSortedCollection do: [:vname |
			menu add: vname toolTip: (env vars at: vname) action: nil]]].

	menu build.
	mh _ menu menuForm height.
	mw _ menu menuForm width.

	menu startUpWithCaption: nil at: (varButton left + (varButton width - mw // 2)) @ (varButton top - mh)! !

!DebuggerFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:51'!
yes
	"Yes button was pressed."

	self rememberUserDimensions.
	super yes.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DebuggerFrameMorph class
	instanceVariableNames: ''!

!DebuggerFrameMorph class methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:55'!
initialize
	"DebuggerFrameMorph initialize"
	self resetDimensions! !

!DebuggerFrameMorph class methodsFor: 'as yet unclassified' stamp: 'jens 6/10/2010 00:12'!
minExtent
	^350 @ 180.! !

!DebuggerFrameMorph class methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:54'!
resetDimensions
	UserPos _ nil.
	UserSize _ self minExtent.! !

!DebuggerFrameMorph class methodsFor: 'as yet unclassified' stamp: 'jens 6/7/2010 21:53'!
setUserPos: aPoint size: anExtent
	UserPos _ aPoint.
	UserSize _ anExtent.! !


DebuggerFrameMorph initialize!
